﻿using System;
using System.Collections.Generic;
using System.Data;
using AutoProofreader.Dao;
using AutoProofreader.Enums;
using AutoProofreader.Exceptions;
using AutoProofreader.Managers;
using AutoProofreader.Utils;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Answer business object implementation 
    /// </summary>
    [Serializable]
    public class Answer : BaseBusinessObject, IBusinessObject
    {
        [NonSerialized]
        private IAnswerDAO answerDao = DaoProvider.GetAnswerDao();

        #region Properties

        private Question question = new Question();

        public Question Question
        {
            get { return question; }
            set { question = value; }
        }


        private int? answerNumber;

        public int? AnswerNumber
        {
            get { return answerNumber; }
            set { answerNumber = value; }
        }

        private string answerDescription;

        public string AnswerDescription
        {
            get { return answerDescription; }
            set { answerDescription = value; }
        }

        private string isCorrectHint;

        public string IsCorrectHint
        {
            get { return isCorrectHint; }
            set { isCorrectHint = value; }
        }

        private string isIncorrectHint;

        public string IsIncorrectHint
        {
            get { return isIncorrectHint; }
            set { isIncorrectHint = value; }
        }
        private string correctValue;

        public string CorrectValue
        {
            get { return correctValue; }
            set { correctValue = value; }
        }
        #endregion

        /// <summary>
        /// Returns whether the answer provided by the student is correct
        /// </summary>
        public bool IsCorrect
        {
            get
            {
                bool isCorrect = false;

                if (bool.TryParse(correctValue, out isCorrect))
                {
                    return isCorrect;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// The answer given by the student on an attempt
        /// </summary>
        private string attemptValue;
        public string AttemptValue
        {
            get { return attemptValue; }
            set { attemptValue = value; }
        }

        /// <summary>
        /// The answer input by the user on UI
        /// </summary>
        private string userValue;
        public string UserValue
        {
            get { return userValue; }
            set { userValue = value; }
        }

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>
        public bool Exists()
        {
            return this.question.QuestionId.HasValue && this.answerNumber.HasValue;
        }

        /// <summary>
        /// Loads the answer by its question and answer number properties
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            if (this.Exists())
            {
                return Load(this.question, this.answerNumber.Value);
            }
            return false;
        }

        /// <summary>
        /// Loads the answer by question and answer number 
        /// </summary>
        /// <param name="question">Answer's question</param>
        /// <param name="answerNumber">Answer number</param>
        /// <returns>True if the answer has been found</returns>
        public bool Load(Question question, long answerNumber)
        {
            IDataReader dataReader = answerDao.GetById(question.QuestionId.Value, answerNumber);
            return Load(dataReader);
        }


        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            if (question.AnswersType.Value == AnswersType.OneCorrect)
            {
                if (!IsTheOnlyCorrectAnswer())
                {
                    throw new BusinessLogicException("Aquesta pregunta ja té una resposta correcta. No pot haver més d'una reposta correcta en aquest tipus de pregunta.");
                }
            }
            if (!Exists())
            {
                this.answerNumber = answerDao.Add(this.question.QuestionId.Value, this.answerDescription, this.isCorrectHint, this.isIncorrectHint, this.correctValue);
            }
            else
            {
                answerDao.Update(this.question.QuestionId.Value, this.answerNumber.Value, this.answerDescription, this.isCorrectHint, this.isIncorrectHint, this.correctValue);
            }
        }

        /// <summary>
        /// Deletes the business object using the underlaying business object primary key
        /// </summary>
        public void Delete()
        {
            answerDao.Delete(this.question.QuestionId.Value, this.answerNumber.Value);
        }

        /// <summary>
        /// Checks if this is the only correct answer on the answer's question
        /// </summary>
        /// <returns>True if there are no more correct answers</returns>
        private bool IsTheOnlyCorrectAnswer()
        {
            if (this.correctValue == bool.TrueString)
            {
                List<Answer> answerList = AnswerManager.ListByQuestion(question);

                foreach (Answer answer in answerList)
                {
                    if (answer.correctValue == bool.TrueString && answer.answerNumber != this.answerNumber)
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.answerDescription;
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>
        public override void FromDataReader(IDataReader dataReader)
        {
            FromDataReader(this, dataReader);
        }

        /// <summary>
        /// Fills business object properties  passed as a parameter from a data reader
        /// </summary>
        /// <param name="answer">The business objet</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(Answer answer, IDataReader dataReader)
        {
            answer.Question.QuestionId = DbUtils.ReadLong(dataReader, "QuestionId");
            answer.AnswerNumber = DbUtils.ReadInt(dataReader, "AnswerNumber");
            answer.AnswerDescription = DbUtils.ReadString(dataReader, "AnswerDescription");
            answer.IsCorrectHint = DbUtils.ReadString(dataReader, "IsCorrectHint");
            answer.IsIncorrectHint = DbUtils.ReadString(dataReader, "IsIncorrectHint");
            answer.CorrectValue = DbUtils.ReadString(dataReader, "CorrectValue");
        }

        /// <summary>
        /// Checks if the answer given by the user matches with the expected answer correct value
        /// </summary>
        /// <returns>True if the user value matches with the expected answer correct value</returns>
        public bool Correct()
        {
            if (!string.IsNullOrEmpty(this.CorrectValue) &&
                this.CorrectValue.Equals(this.userValue, StringComparison.InvariantCultureIgnoreCase))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
