﻿using System;
using System.Data;
using System.Data.Common;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AutoProofreader.Utils;
using AutoProofreader.Dao;
using AutoProofreader.Exceptions;
using AutoProofreader.Enums;
using AutoProofreader.Managers;
using AutoProofreader.Security;
using AutoProofreader.Email;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Attempt business object implementation 
    /// </summary>
    [Serializable]
    public class Attempt : BaseBusinessObject
    {
        [NonSerialized]
        private IAttemptDAO attemptDao = DaoProvider.GetAttemptDao();

        #region Properties

        private long? attemptId;

        public long? AttemptId
        {
            get { return attemptId; }
            set { attemptId = value; }
        }

        private Classroom classroom;

        public Classroom Classroom
        {
            get { return classroom; }
            set { classroom = value; }
        }

        private Test test;
        public Test Test
        {
            get { return test; }
            set { test = value; }
        }

        private User user = new User();
        public User User
        {
            get { return user; }
            set { user = value; }
        }

        private DateTime? startDate = new DateTime();
        public DateTime? StartDate
        {
            get { return startDate; }
            set { startDate = value; }
        }

        private DateTime? endDate = null;
        public DateTime? EndDate
        {
            get { return endDate; }
            set { endDate = value; }
        }

        private string score;
        public string Score
        {
            get
            {
                if (string.IsNullOrEmpty(score) &&
                    HasExpired())
                {
                    return test.GetScoreEvaluationType(0);
                }
                return score;
            }
            set { score = value; }
        }
        #endregion

        /// <summary>
        /// Time taken by the student to complete the attempt
        /// </summary>
        public TimeSpan ElapsedTime
        {
            get
            {
                if (this.endDate.HasValue && this.startDate.HasValue)
                {
                    return this.endDate.Value - this.startDate.Value;
                }
                else
                {
                    if (this.startDate.HasValue)
                    {
                        return TimeSpan.FromMinutes((double)this.Test.ResolutionMaxMinutes);
                    }
                    else
                    {
                        return new TimeSpan(0);
                    }
                }
            }
        }

        /// <summary>
        ///  Time taken by the student to complete the attempt, as a string
        /// </summary>
        public string ElapsedTimeToString
        {
            get
            {
                if (!IsStillGoingOn)
                {
                    return DateUtils.TimeSpanToString(ElapsedTime);
                }
                else
                {
                    return "";
                }
            }
        }

        /// <summary>
        /// Checks if the student is still completing the test
        /// </summary>
        public bool IsStillGoingOn
        {
            get
            {
                if (this.startDate.HasValue && !this.endDate.HasValue)
                {
                    if (this.startDate.HasValue)
                    {
                        return this.startDate.Value.AddMinutes((double)this.Test.ResolutionMaxMinutes) > DateTime.Now;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// Default constructor
        /// </summary>
        public Attempt()
            : base()
        {

        }

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>

        public bool Exists()
        {
            return this.attemptId.HasValue;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            bool found = false;

            if (this.Exists())
            {
                found = Load(this.attemptId.Value);
                if (this.test.Load())
                {
                    this.test.FillQuestionsByAttempt(this);
                }
            }
            return found;
        }


        /// <summary>
        /// Loads the business object by a primary key
        /// </summary>
        /// <param name="attemptId">primary key value</param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long attemptId)
        {
            if (this.Exists())
            {
                IDataReader dataReader = attemptDao.GetById(this.attemptId.Value);
                return base.Load(dataReader);
            }
            return false;
        }

        /// <summary>
        /// Updates the attempt question answers given by the student to the database
        /// </summary>
        public void UpdateAttemptQuestionAnswer()
        {
            using (DaoTransaction transaction = new DaoTransaction())
            {
                try
                {
                    UpdateAttemptQuestionAnswer(transaction);
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    throw;
                }
            }
        }
        /// <summary>
        /// Updates the attempt question answers given by the student to the database
        /// </summary>
        /// <param name="transaction">Transaction to use</param>

        public void UpdateAttemptQuestionAnswer(DaoTransaction transaction)
        {
            this.attemptDao = DaoProvider.GetAttemptDao();

            foreach (Question question in this.test.QuestionList)
            {
                foreach (Answer answer in question.AnswerList)
                {
                    if (answer.UserValue != answer.AttemptValue)
                    {
                        if (answer.AttemptValue == null)
                        {
                            attemptDao.AddQuestionAnswer(transaction, this.attemptId.Value, question.QuestionId.Value, answer.AnswerNumber.Value, answer.UserValue);
                        }
                        else
                        {
                            attemptDao.UpdateQuestionAnswer(transaction, this.attemptId.Value, question.QuestionId.Value, answer.AnswerNumber.Value, answer.UserValue);
                        }
                        answer.AttemptValue = answer.UserValue;
                    }
                }
            }
        }

        /// <summary>
        /// Updates the attempt question given by the student to the database
        /// </summary>
        public void UpdateAttemptQuestion(DaoTransaction transaction)
        {
            this.attemptDao = DaoProvider.GetAttemptDao();

            foreach (Question question in this.test.QuestionList)
            {
                attemptDao.UpdateQuestion(transaction, this.attemptId.Value, question.QuestionId.Value, question.Result.Value);
            }
        }

        /// <summary>
        /// Checks if the student has exceed the maximum number of attempts allowed by the test
        /// </summary>
        /// <returns>True if the student has exceed the maximum number of attempts allowed by the test</returns>
        public bool HasExceedeedNumberOfAttempts()
        {
            //Check if the user hasn't reached max number of attemps.
            if (this.Classroom != null && this.Classroom.Exists())
            {
                int numberOfAttempts = AttemptManager.GetNumberOfAttempts(this.Test, this.Classroom, user);
                if (numberOfAttempts >= this.test.MaxAttempts)
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Starts the attempt by adding it and its questions to the database
        /// </summary>
        public void Start()
        {
            if (test.Exists())
            {
                this.startDate = DateTime.Now;
                this.user = new User(new SecurityProvider().GetCurrentUserSession());

                //If there's a classroom selected, we need to save the attempt
                if (classroom.Exists())
                {
                    using (DaoTransaction transaction = new DaoTransaction())
                    {
                        try
                        {
                            this.attemptDao = DaoProvider.GetAttemptDao();
                            this.attemptId = attemptDao.Add(transaction, this.classroom.ClassroomId.Value, this.test.TestId.Value, this.user.UserId.Value, startDate.Value, null, null);
                            this.AddQuestions(transaction);
                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                            throw;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Stops the student attempt.
        /// Corrects the test, update the attempt to the database and its questions.
        /// </summary>
        public void Stop()
        {
            if (test.Exists())
            {
                this.endDate = DateTime.Now;
                this.score = test.Correct();

                if (classroom.Exists())
                {
                    using (DaoTransaction transaction = new DaoTransaction())
                    {
                        try
                        {
                            this.attemptDao = DaoProvider.GetAttemptDao();
                            attemptDao.Update(transaction, this.attemptId.Value, this.endDate.Value, this.score);
                            UpdateAttemptQuestion(transaction);
                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                            throw;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Checks whether the attempt has expired.
        /// </summary>
        /// <returns>True if the attempt expired</returns>
        public bool HasExpired()
        {
            if (this.startDate.HasValue)
            {
                if (this.test.Exists())
                {
                    if (!this.endDate.HasValue)
                    {
                        if (this.test.ResolutionMaxMinutes.HasValue)
                        {
                            if (DateTime.Now >= this.startDate.Value.Add(new TimeSpan(0, this.test.ResolutionMaxMinutes.Value, 0)))
                            {
                                return true;
                            }
                        }
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            return false;
        }


        public override void FromDataReader(IDataReader dataReader)
        {
            FromDataReader(this, dataReader);
        }


        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="attempt">The attempt</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(Attempt attempt, IDataReader dataReader)
        {
            attempt.AttemptId = DbUtils.ReadLong(dataReader, "AttemptId");

            attempt.Classroom = new Classroom();
            attempt.Classroom.ClassroomId = DbUtils.ReadLong(dataReader, "ClassroomId");

            attempt.Test = new Test();
            attempt.Test.TestId = DbUtils.ReadLong(dataReader, "TestId");

            attempt.User = new User();
            attempt.User.UserId = DbUtils.ReadLong(dataReader, "UserId");
            attempt.StartDate = DbUtils.ReadDate(dataReader, "StartDate");
            attempt.EndDate = DbUtils.ReadDate(dataReader, "EndDate");
            attempt.Score = DbUtils.ReadString(dataReader, "Score");

        }

        /// <summary>
        /// Adds questions to the atttempt
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        private void AddQuestions(DaoTransaction transaction)
        {
            foreach (Question question in test.QuestionList)
            {
                attemptDao.AddQuestion(transaction, this.attemptId.Value, question.QuestionId.Value, null);
            }
        }

    }
}
