﻿using System;
using System.Data;
using AutoProofreader.Dao;
using AutoProofreader.Utils;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Category business object implementation 
    /// </summary>
    [Serializable]
    public class Category : BaseBusinessObject, IBusinessObject
    {
        [NonSerialized]
        private ICategoryDAO categoryDao = DaoProvider.GetCategoryDao();

        #region Properties

        private long? categoryId;

        public long? CategoryId
        {
            get { return categoryId; }
            set { categoryId = value; }
        }
        private string categoryName;

        public string CategoryName
        {
            get { return categoryName; }
            set { categoryName = value; }
        }

        #endregion

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>        
        public bool Exists()
        {
            return this.categoryId.HasValue;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            if (this.Exists())
            {
                return Load(this.categoryId.Value);
            }
            return false;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="categoryId">Primary key</param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long categoryId)
        {
            IDataReader dataReader = categoryDao.GetById(categoryId);
            return Load(dataReader);
        }

        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            using (DaoTransaction transaction = new DaoTransaction())
            {
                try
                {
                    if (!Exists())
                    {
                        this.IsDeleted = false;

                        //Create category
                        this.categoryId = categoryDao.Add(transaction, this.categoryName, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                        //Log message
                        Log.Add(transaction, string.Format("Ha creat la categoria {0}", this.ToString()));
                    }
                    else
                    {
                        //update category
                        categoryDao.Update(transaction, this.categoryId.Value, this.categoryName, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                        //Log message
                        Log.Add(transaction, string.Format("Ha modificat categoria {0}", this.ToString()));
                    }

                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    throw;
                }
            }

        }

        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.categoryName;
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>

        public override void FromDataReader(IDataReader dataReader)
        {
            FromDataReader(this, dataReader);
        }


        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="category">Business objet</param>
        /// <param name="dataReader">the data reader to read from</param>

        public static void FromDataReader(Category category, IDataReader dataReader)
        {
            category.categoryId = DbUtils.ReadLong(dataReader, "CategoryId");
            category.CategoryName = DbUtils.ReadString(dataReader, "CategoryName");

            BaseBusinessObject.FromDataReader(category, dataReader);
        }

    }
}
