﻿using System;
using System.Data;
using AutoProofreader.Dao;
using AutoProofreader.Exceptions;
using AutoProofreader.Utils;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Classroom business object implementation 
    /// </summary>
    [Serializable]
    public class Classroom : BaseBusinessObject
    {
        [NonSerialized]
        private IClassroomDAO classroomDao = DaoProvider.GetClassroomDao();

        #region Properties

        private long? classroomId;

        public long? ClassroomId
        {
            get { return classroomId; }
            set { classroomId = value; }
        }
        private string classroomName;

        public string ClassroomName
        {
            get { return classroomName; }
            set { classroomName = value; }
        }

        private DateTime? startDate;

        public DateTime? StartDate
        {
            get { return startDate; }
            set { startDate = value; }
        }

        private DateTime? endDate;

        public DateTime? EndDate
        {
            get { return endDate; }
            set { endDate = value; }
        }



        #endregion

        #region Support properties
        private long? studentCount;

        public long? StudentCount
        {
            get { return studentCount; }
            set { studentCount = value; }
        }
        private long? teacherCount;

        public long? TeacherCount
        {
            get { return teacherCount; }
            set { teacherCount = value; }
        }

        private DateTime? testStartDate;

        public DateTime? TestStartDate
        {
            get { return testStartDate; }
            set { testStartDate = value; }
        }

        private DateTime? testEndDate;

        public DateTime? TestEndDate
        {
            get { return testEndDate; }
            set { testEndDate = value; }
        }
        #endregion


        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>
        public bool Exists()
        {
            return this.classroomId.HasValue;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            if (this.Exists())
            {
                return Load(this.classroomId.Value);
            }
            return false;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="classroomId">Primary key</param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long classroomId)
        {
            IDataReader dataReader = classroomDao.GetById(classroomId);
            return Load(dataReader);
        }


        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            using (DaoTransaction transaction = new DaoTransaction())
            {
                try
                {
                    if (!Exists())
                    {
                        this.IsDeleted = false;
                        //Add classroom
                        this.ClassroomId = classroomDao.Add(transaction, this.classroomName, this.startDate, this.endDate, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                        //Log
                        Log.Add(transaction, string.Format("Ha creat l'aula {0}", this.ToString()));

                    }
                    else
                    {
                        //Update classroom
                        classroomDao.Update(transaction, this.ClassroomId.Value, this.classroomName, this.startDate, this.endDate, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                        //Log
                        Log.Add(transaction, string.Format("Ha modificat l'aula {0}", this.ToString()));
                    }

                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    throw;
                }
            }
        }


        /// <summary>
        /// Assign a classroom to a user
        /// </summary>
        /// <param name="user">The user</param>
        public void AssignUser(User user)
        {
            if (Exists())
            {
                using (DaoTransaction transaction = new DaoTransaction())
                {
                    try
                    {
                        //Assign 
                        classroomDao.AssignUser(this.classroomId.Value, user.UserId.Value);

                        //Log
                        Log.Add(string.Format("Ha assignat l'aula {0} a l'usuari {1}", this.ToString(), user.ToString()));

                        transaction.Commit();
                    }
                    catch (Exception)
                    {
                        transaction.Rollback();
                        throw;
                    }
                }
            }
            else
            {
                throw new BusinessLogicException("");
            }
        }

        /// <summary>
        /// Unassign a  user to a classroom
        /// </summary>
        /// <param name="user">The user</param>
        public void UnassignUser(User user)
        {
            if (Exists())
            {
                using (DaoTransaction transaction = new DaoTransaction())
                {
                    try
                    {
                        //Unassign
                        classroomDao.UnassignUser(this.classroomId.Value, user.UserId.Value);

                        //Log
                        Log.Add(string.Format("Ha desassignat l'aula {0} de l'usuari {1}", this.ToString(), user.ToString()));

                        transaction.Commit();
                    }
                    catch (Exception)
                    {
                        transaction.Rollback();
                        throw;
                    }
                }
            }
            else
            {
                throw new BusinessLogicException("");
            }
        }


        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.classroomName;
        }


        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>

        public override void FromDataReader(IDataReader dataReader)
        {
            Classroom.FromDataReader(this, dataReader);
        }


        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="classroom">Business object</param>
        /// <param name="dataReader">the data reader to read from</param>

        public static void FromDataReader(Classroom classroom, IDataReader dataReader)
        {
            classroom.classroomId = DbUtils.ReadLong(dataReader, "ClassroomId");
            classroom.classroomName = DbUtils.ReadString(dataReader, "ClassroomName");
            classroom.startDate = DbUtils.ReadDate(dataReader, "StartDate");
            classroom.endDate = DbUtils.ReadDate(dataReader, "EndDate");
            classroom.CreatedDate = DbUtils.ReadDate(dataReader, "CreatedDate");

            BaseBusinessObject.FromDataReader(classroom, dataReader);
        }


    }
}
