﻿using System;
using System.Data;
using AutoProofreader.Dao;
using AutoProofreader.Enums;
using AutoProofreader.Utils;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Log business object implementation 
    /// </summary>
    [Serializable]
    public class Log : BaseBusinessObject
    {
        [NonSerialized]
        private ILogDAO logDao = DaoProvider.GetLogDao();

        #region Properties

        private long? logId;

        public long? LogId
        {
            get { return logId; }
            set { logId = value; }
        }

        private string description;

        public string Description
        {
            get { return description; }
            set { description = value; }
        }


        private LogType logType;

        public LogType LogType
        {
            get { return logType; }
            set { logType = value; }
        }
        #endregion

        public string UserName
        {
            get
            {
                if (this.createdBy != null)
                {
                    return this.createdBy.UserName;
                }
                else
                {
                    return "";
                }
            }
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>
        public override void FromDataReader(IDataReader dataReader)
        {
            Log.FromDataReader(this, dataReader);
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="log">Business objet</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(Log log, IDataReader dataReader)
        {
            log.logId = DbUtils.ReadLong(dataReader, "LogId");
            log.description = DbUtils.ReadString(dataReader, "Description");
            log.createdDate = DbUtils.ReadDate(dataReader, "CreatedDate");
            log.logType = LogType.FromString(DbUtils.ReadString(dataReader, "LogType"));
        }

        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        private void Save(DaoTransaction transaction)
        {
            this.logId = logDao.Add(transaction, this.description, this.logType.ToValue(), DateTime.Now, securityProvider.GetCurrentUserSession().UserId);
        }

        /// <summary>
        /// Adds a new log of type info 
        /// </summary>
        /// <param name="description">Log message</param>
        public static void Add(string description)
        {
            Add(new LogType(LogType.Info), description);
        }

        /// <summary>
        /// Adds a new log of type info 
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        /// <param name="description">Log message</param>
        public static void Add(DaoTransaction transaction, string description)
        {
            Add(transaction, new LogType(LogType.Info), description);
        }

        /// <summary>
        /// Adds a new log to the database
        /// </summary>
        /// <param name="logType">Type of the log mesage </param>
        /// <param name="description">Log message</param>
        public static void Add(LogType logType, string description)
        {
            Add(null, logType, description);
        }

        /// <summary>
        /// Adds a new log to the database
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        /// <param name="logType">Type of the log mesage </param>
        /// <param name="description">Log message</param>
        public static void Add(DaoTransaction transaction, LogType logType, string description)
        {
            Log log = new Log();
            log.LogType = logType;
            log.description = description;
            log.Save(transaction);
        }


        /// <summary>
        /// Adds a new log of type error to the database
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        /// <param name="description">Log message</param>
        /// <param name="ex">The exception we want to use</param>
        public static void Error(DaoTransaction transaction, string description, Exception ex)
        {
            Add(transaction, new LogType(LogType.Error), description + Environment.NewLine + "-Error:" + ex.Message);
        }

        /// <summary>
        /// Adds a new log of type error to the database
        /// </summary>
        /// <param name="description">Log message</param>
        /// <param name="ex">The exception we want to use</param>

        public static void Error(string description, Exception ex)
        {
            string errorMessage = description;
            if (ex != null)
            {
                errorMessage += Environment.NewLine + "- Error:" + ex.Message;
            }
            Add(null, new LogType(LogType.Error), errorMessage);
        }


        /// <summary>
        /// Adds a new log of type error to the database
        /// </summary>
        /// <param name="description">Log message</param>
        public static void Error(string description)
        {
            Add(null, new LogType(LogType.Error), description);
        }


        /// <summary>
        /// Adds a new log of type warning to the database
        /// </summary>
        /// <param name="description">Log message</param>

        public static void Warning(string description)
        {
            Add(null, new LogType(LogType.Warning), description);
        }

        /// <summary>
        /// Adds a new log of type info to the database
        /// </summary>
        /// <param name="description">Log message</param>
        public static void Info(string description)
        {
            Info(null, description);
        }

        /// <summary>
        /// Adds a new log of type warning to the database
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        /// <param name="description">Log message</param>

        public static void Info(DaoTransaction transaction, string description)
        {
            Add(transaction, new LogType(LogType.Info), description);
        }
    }
}
