﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using AutoProofreader.Dao;
using AutoProofreader.Enums;
using AutoProofreader.Managers;
using AutoProofreader.Resources;
using AutoProofreader.Utils;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Question business object implementation 
    /// </summary>
    [Serializable]
    public class Question : BaseBusinessObject, IBusinessObject
    {
        [NonSerialized]
        private IQuestionDAO questionDao = DaoProvider.GetQuestionDao();

        #region Properties

        private long? questionId;

        public long? QuestionId
        {
            get { return questionId; }
            set { questionId = value; }
        }

        private string questionDescription;

        public string QuestionDescription
        {
            get { return questionDescription; }
            set { questionDescription = value; }
        }

        private AnswersType answersType;
        public AnswersType AnswersType
        {
            get { return answersType; }
            set { answersType = value; }
        }

        private Category category = new Category();
        public Category Category
        {
            get { return category; }
            set { category = value; }
        }

        private EditStatus editStatus = new EditStatus();
        public EditStatus EditStatus
        {
            get { return editStatus; }
            set { editStatus = value; }
        }

        private string imageResource;
        public string ImageResource
        {
            get { return imageResource; }
            set { imageResource = value; }
        }

        private string soundResource;
        public string SoundResource
        {
            get { return soundResource; }
            set { soundResource = value; }
        }
        #endregion

        #region Tests_Questions properties
        private int? sortOrder;
        public int? SortOrder
        {
            get { return sortOrder; }
            set { sortOrder = value; }
        }

        private long? testQuestionId;
        public long? TestQuestionId
        {
            get { return testQuestionId; }
            set { testQuestionId = value; }
        }

        private QuestionType questionType = new QuestionType();
        public QuestionType QuestionType
        {
            get { return questionType; }
            set { questionType = value; }
        }

        #endregion

        private List<Answer> answerList = null;
        /// <summary>
        /// Question answers
        /// </summary>
        public List<Answer> AnswerList
        {
            get
            {
                return this.answerList;
            }
        }

        float? result;
        /// <summary>
        /// Question result
        /// </summary>
        public float? Result
        {
            get
            {
                return this.result;
            }
        }

        /// <summary>
        /// Gets short question description, html characters are cleaned up
        /// </summary>
        public string ShortQuestionDescription
        {
            get
            {
                string shortQuestionDescription = EncodingUtils.CleanEncodedHtml(this.questionDescription);
                if (shortQuestionDescription.Length > 45)
                {
                    shortQuestionDescription = shortQuestionDescription.Substring(0, 45) + "(...)";
                }
                return shortQuestionDescription;
            }
        }

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>
        public bool Exists()
        {
            return this.questionId.HasValue;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            if (this.Exists())
            {
                return Load(this.questionId.Value);
            }
            return false;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="questionId">Primary key</param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long questionId)
        {
            IDataReader dataReader = questionDao.GetById(questionId);
            return base.Load(dataReader);
        }

        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            Save(null);
        }

        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        /// <param name="test">The question will be added to this test once is saved</param>        
        public void Save(Test test)
        {
            if (!Exists())
            {
                this.editStatus = new EditStatus(EditStatus.Edit);
                this.isDeleted = false;
                this.questionType = new QuestionType(QuestionType.New);

                if (test != null)
                {
                    using (DaoTransaction transaction = new DaoTransaction())
                    {
                        try
                        {
                            this.questionId = questionDao.Add(transaction, this.questionDescription, editStatus.ToValue(), answersType.ToValue(), category.CategoryId.Value, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);
                            test.AssignQuestion(this);

                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                            throw;
                        }
                    }
                }
                else
                {
                    this.questionId = questionDao.Add(null, this.questionDescription, questionType.ToValue(), answersType.ToValue(), category.CategoryId.Value, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);
                }
            }
            else
            {
                questionDao.Update(this.questionId.Value, this.questionDescription, this.category.CategoryId.Value, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);
            }
        }

        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.questionDescription;
        }



        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>
        public override void FromDataReader(IDataReader dataReader)
        {
            FromDataReader(this, dataReader);
        }



        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="question">Business object</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(Question question, IDataReader dataReader)
        {
            if (!dataReader.IsDBNull(dataReader.GetOrdinal("QuestionId")))
            {
                question.QuestionId = DbUtils.ReadLong(dataReader, "QuestionId");
                question.QuestionDescription = DbUtils.ReadString(dataReader, "QuestionDescription");
                question.AnswersType = AnswersType.FromString(DbUtils.ReadString(dataReader, "AnswersType"));
                question.Category.CategoryId = DbUtils.ReadLong(dataReader, "CategoryId");
                question.EditStatus = EditStatus.FromString(DbUtils.ReadString(dataReader, "EditStatus"));

                if (question.EditStatus.Value == EditStatus.Edit)
                {
                    question.questionType = new QuestionType(QuestionType.New);
                }
                else
                {
                    question.questionType = new QuestionType(QuestionType.Library);
                }

                question.imageResource = DbUtils.ReadString(dataReader, "ImageResource");
                question.soundResource = DbUtils.ReadString(dataReader, "SoundResource");
            }
            else
            {
                question.questionType = new QuestionType(QuestionType.Random);
            }
        }

        /// <summary>
        /// Validates the question by applying question business logic and returning the validation result.
        /// </summary>
        /// <returns>An array of strings containing the validation result</returns>
        public List<string> GetValidationResult()
        {
            List<string> validationResult = new List<string>();

            if (this.answerList == null)
            {
                LoadAnswers();
            }

            if (answerList.Count > 0)
            {
                if (this.answersType.Value == AnswersType.OneCorrect ||
                    this.answersType.Value == AnswersType.MultipleCorrect)
                {
                    bool correctAnswerFound = false;
                    foreach (Answer answer in answerList)
                    {
                        if (answer.IsCorrect) correctAnswerFound = true;
                    }
                    if (!correctAnswerFound)
                    {
                        validationResult.Add(string.Format("La pregunta '{0}' no té cap resposta correcta.", this.ShortQuestionDescription));
                    }
                }
                else
                {
                    Answer freeTextAnswer = answerList[0];
                    if (string.IsNullOrEmpty(freeTextAnswer.CorrectValue))
                    {
                        validationResult.Add(string.Format("No has indicat quina és la resposta correcta en la pregunta de text lliure '{0}'.", this.ShortQuestionDescription));
                    }
                }
            }
            else
            {
                validationResult.Add(string.Format("La pregunta '{0}' no té cap resposta.", this.ShortQuestionDescription));
            }

            return validationResult;
        }

        /// <summary>
        /// Loads question answers
        /// </summary>
        public void LoadAnswers()
        {
            this.answerList = AnswerManager.ListByQuestion(this);
        }

        /// <summary>
        /// Corrects the question. Result is stored on <see cref="Result"/> property
        /// </summary>
        public void Correct()
        {
            this.result = 0;

            if (this.answersType.Value == AnswersType.FreeText)
            {
                if (this.answerList.Count > 0)
                {
                    Answer answer = this.AnswerList[0];
                    if (answer.Correct())
                    {
                        this.result = 1;
                    }
                }
            }
            else
            {
                float numberOfCorrectAnswers;
                if (this.answersType.Value == AnswersType.OneCorrect)
                {
                    numberOfCorrectAnswers = 1;
                }
                else
                {
                    numberOfCorrectAnswers = GetNumberOfCorrectAnswers();
                }

                if (numberOfCorrectAnswers > 0 && answerList != null)
                {
                    foreach (Answer answer in answerList)
                    {
                        if (answer.Correct())
                        {
                            this.result += 1 / numberOfCorrectAnswers;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets how many correct ansewers holds the question
        /// </summary>
        /// <returns>Number of correct answers</returns>
        private float GetNumberOfCorrectAnswers()
        {
            float numberOfCorrectAnswers = 0;
            foreach (Answer answer in answerList)
            {
                if (answer.IsCorrect) numberOfCorrectAnswers++;
            }

            return numberOfCorrectAnswers;
        }

        /// <summary>
        /// Adds an attempt answer to the question
        /// </summary>
        /// <param name="answer">The answer</param>
        public void AddAttemptAnswer(Answer answer)
        {
            if (this.answerList == null) this.answerList = new List<Answer>();
            this.answerList.Add(answer);
        }

        /// <summary>
        /// Save multimedia files to the question, files are stored using <see cref="ResourceProvider"/> provider
        /// </summary>
        /// <param name="imageName">Name of the image file</param>
        /// <param name="imageStream">Image stream</param>
        /// <param name="soundName">Name of the sound file</param>
        /// <param name="soundStream">Sound steam</param>
        public void SaveMultimedia(string imageName, Stream imageStream, string soundName, Stream soundStream)
        {
            //Save image
            if (imageStream != null && imageStream.Length > 0)
            {
                this.imageResource = imageName;
                ResourceProvider.UploadImage(this.QuestionId.ToString(), imageName, imageStream);
            }

            //Save sound
            if (soundStream != null && soundStream.Length > 0)
            {
                this.soundResource = soundName;
                ResourceProvider.UploadSound(this.QuestionId.ToString(), soundName, soundStream);
            }

            questionDao.UpdateResources(this.questionId.Value, this.imageResource, this.soundResource, DateTime.Now, securityProvider.GetCurrentUserSession().UserId);

        }

        /// <summary>
        /// Removes multimedia from the question
        /// </summary>
        /// <param name="removeImage">If true, will remove question image</param>
        /// <param name="removeSound">If true, will remove question sound</param>
        public void RemoveMultimedia(bool removeImage, bool removeSound)
        {
            if (removeImage && this.ExistImage())
            {
                ResourceProvider.RemoveImage(this.QuestionId.ToString(), this.imageResource);
                this.imageResource = null;
            }

            if (removeSound && this.ExistSound())
            {
                ResourceProvider.RemoveSound(this.QuestionId.ToString(), this.soundResource);
                this.soundResource = null;
            }

            questionDao.UpdateResources(this.questionId.Value, this.imageResource, this.soundResource, DateTime.Now, securityProvider.GetCurrentUserSession().UserId);
        }

        /// <summary>
        /// Checks whether the question holds any image
        /// </summary>
        /// <returns>True if the question has an image </returns>
        public bool ExistImage()
        {
            return !string.IsNullOrEmpty(this.imageResource);
        }

        /// <summary>
        /// Gets the path to the question image resource
        /// </summary>
        /// <returns>path to the question image resource</returns>
        public string GetImagePath()
        {
            if (ExistImage())
            {
                return ResourceProvider.GetImagePath(this.QuestionId.ToString(), this.imageResource);
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Checks whether the question holds any sound
        /// </summary>
        /// <returns>True if the question has a sound</returns>
        public bool ExistSound()
        {
            return !string.IsNullOrEmpty(this.soundResource);
        }

        /// <summary>
        /// Gets the path to the soundimage resource
        /// </summary>
        /// <returns>path to the question sound resource</returns>        
        public string GetSoundPath()
        {
            if (ExistSound())
            {
                return ResourceProvider.GetSoundPath(this.QuestionId.ToString(), this.soundResource);
            }
            else
            {
                return null;
            }
        }
    }
}
