﻿using System;
using System.Data;
using System.Data.Common;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AutoProofreader.Utils;
using AutoProofreader.Managers;
using AutoProofreader.Dao;
using AutoProofreader.Exceptions;
using AutoProofreader.Enums;
using AutoProofreader.Security;
using AutoProofreader.Email;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// Test business object implementation 
    /// </summary>
     [Serializable]
    public class Test : BaseBusinessObject
    {
        [NonSerialized]
        private ITestDAO testDao = DaoProvider.GetTestDao();

        #region Properties
         
        private long? testId;

        public long? TestId
        {
            get { return testId; }
            set { testId = value; }
        }

        private string testName;

        public string TestName
        {
            get { return testName; }
            set { testName = value; }
        }


        private string testDescription;

        public string TestDescription
        {
            get { return testDescription; }
            set { testDescription = value; }
        }

        private int? maxAttempts;

        public int? MaxAttempts
        {
            get { return maxAttempts; }
            set { maxAttempts = value; }
        }

        private int? questionsPerPage;

        public int? QuestionsPerPage
        {
            get { return questionsPerPage; }
            set { questionsPerPage = value; }
        }

        private int? resolutionMaxMinutes;

        public int? ResolutionMaxMinutes
        {
            get { return resolutionMaxMinutes; }
            set { resolutionMaxMinutes = value; }
        }

        private EvaluationType evaluationType;
        public EvaluationType EvaluationType
        {
            get { return evaluationType; }
            set { evaluationType = value; }
        }

        private EditStatus editStatus;
        public EditStatus EditStatus
        {
            get { return editStatus; }
            set { editStatus = value; }
        }
        #endregion 


        private List<Question> questionList = null;
         /// <summary>
         /// Test questions
         /// </summary>
        public List<Question> QuestionList
        {
            get
            {
                return this.questionList;
            }
        }

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>
        public bool Exists()
        {
            return this.testId.HasValue;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>
        public bool Load()
        {
            if (this.Exists())
            {
                return Load(this.testId.Value);
            }
            return false;
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="loadQuestions">if true, test questions will be retrieved from database </param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(bool loadQuestions)
        {
            if (this.Exists())
            {
                return Load(this.testId.Value, loadQuestions);
            }
            return false;
        }


        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="testId">Primary key</param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long testId)
        {
            return Load(testId, false);
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <param name="testId">Primary key</param>
        /// <param name="loadQuestions">if true, test questions will be retrieved from database </param>
        /// <returns>True if the business object has been found</returns>
        public bool Load(long testId, bool loadQuestions)
        {
            IDataReader dataReader = testDao.GetById(testId);
            bool found = base.Load(dataReader);

            if (found && loadQuestions) 
            {
                FillQuestions();
            }
            return found;
        }


        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        /// <param name="transaction">Transaction to use</param>
        internal void Save(DaoTransaction transaction)
        {
            if (!Exists())
            {
                this.isDeleted = false;

                this.editStatus = new EditStatus(EditStatus.Edit);

                //Add test
                this.testId = testDao.Add(transaction, this.testName, this.testDescription, this.maxAttempts, this.questionsPerPage, this.resolutionMaxMinutes, this.evaluationType.ToValue(), this.editStatus.ToValue(), DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                //Log
                Log.Add(transaction, string.Format("Ha creat el test {0}", this.ToString()));
            }
            else
            {
                //Update test
                testDao.Update(transaction, this.testId.Value, this.testName, this.testDescription, this.maxAttempts, this.questionsPerPage, this.resolutionMaxMinutes, this.evaluationType.ToValue(), this.editStatus.ToValue(), DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                //Log
                Log.Add(transaction, string.Format("Ha modificat el test {0}", this.ToString()));
            }
        }

        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            using (DaoTransaction transaction = new DaoTransaction())
            {

                try
                {
                    Save(transaction);

                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    throw;
                }           
            }

        }

        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.testName;
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>
        public override void FromDataReader(IDataReader dataReader)
        {
            Test.FromDataReader(this, dataReader);

            BaseBusinessObject.FromDataReader(this, dataReader);
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="test">Business object</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(Test test, IDataReader dataReader)
        {
            test.TestId = DbUtils.ReadLong(dataReader, "TestId");
            test.TestName = DbUtils.ReadString(dataReader, "TestName");
            test.TestDescription = DbUtils.ReadString(dataReader, "TestDescription");
            test.MaxAttempts = DbUtils.ReadInt(dataReader, "MaxAttempts");
            test.QuestionsPerPage = DbUtils.ReadInt(dataReader, "QuestionsPerPage");
            test.ResolutionMaxMinutes = DbUtils.ReadInt(dataReader, "ResolutionMaxMinutes");
            test.EditStatus = EditStatus.FromString(DbUtils.ReadString(dataReader, "EditStatus"));
            test.EvaluationType = EvaluationType.FromString(DbUtils.ReadString(dataReader, "EvaluationType"));
        }

        /// <summary>
        /// Assign a list of question to the test
        /// </summary>
        /// <param name="questions">List of questions</param>
        public void AssignQuestion(List<Question> questions)
        {
            using (DaoTransaction transaction = new DaoTransaction())
            {
                try
                {
                    foreach (Question question in questions)
                    {
                        testDao.AddQuestion(transaction, this.testId.Value, question.QuestionId.Value, null);
                    }
                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();

                    throw;
                }
            }
        }

        /// <summary>
        /// Assign a question to the test
        /// </summary>
        /// <param name="question">The question</param>
        public void AssignQuestion(Question question)
        {
            testDao.AddQuestion(null, this.testId.Value, question.QuestionId.Value, null);
        }

        /// <summary>
        /// Assign a random question of a category to the test
        /// </summary>
        /// <param name="cateogry">The category</param>
        public void AssignQuestion(Category category)
        {
            testDao.AddQuestion(null, this.testId.Value, null, category.CategoryId.Value);
        }

         /// <summary>
         /// Unassign a question from the test
         /// </summary>
         /// <param name="question">the question</param>
        public void UnassignQuestion(Question question)
        {
            testDao.RemoveQuestion(question.TestQuestionId.Value);
        }

         /// <summary>
         /// Adds the test to a classroom
         /// </summary>
         /// <param name="classroom">The classroom</param>
         /// <param name="startDate">The test will be available since this date</param>
        /// <param name="endDate">The test will be available until this date</param>
        public void AddToClassroom(Classroom classroom, DateTime startDate, DateTime endDate)
        {
            testDao.AddClassroom(this.testId.Value, classroom.ClassroomId.Value, startDate, endDate);
        }

         /// <summary>
         /// Gets the questions of the test
         /// </summary>
        public void GenerateQuestions()
        {
            this.questionList = QuestionManager.GenerateForTest(this);
        }

         /// <summary>
         /// Gets test questions from a given attempt 
         /// </summary>
         /// <param name="attempt">The attempt</param>
        public void FillQuestionsByAttempt(Attempt attempt)
        {
            this.questionList = QuestionManager.ListByAttempt(attempt);
        }

         /// <summary>
         /// Validates the test according to the domain business logic and returns the result on a list
         /// </summary>
         /// <returns>List of strings containing validation results. An empty list points to a correct test</returns>
        public List<string> GetValidationResult()
        {
            List<string> validationResult = new List<string>();

            if (questionList == null) FillQuestions();

            if (questionList.Count > 0)
            {
                foreach (Question question in questionList)
                {                    
                    if (question.QuestionType.Value == QuestionType.New) 
                    {
                        validationResult.AddRange(question.GetValidationResult()); 
                    }                   
                    else
                    {
                        if (question.QuestionType.Value == QuestionType.Random)  
                        {
                            if (QuestionManager.ListFromLibrary(question.Category).Count == 0)
                            {
                                validationResult.Add(string.Format("La prova conté pregunta(es) aleatòries de la categoria {0} però no hi ha cap pregunta amb aquesta categoria a la biblioteca de preguntes.", question.Category.CategoryName));
                            }
                        }
                    }
                }
            }
            else
            {
                validationResult.Add("Aquest test no es pot publicar perquè no té cap pregunta.");
            }


            return validationResult;
        }

        /// <summary>
        /// Fills test questions
        /// </summary>
        private void FillQuestions()
        {
            this.questionList = QuestionManager.ListByTest(this);
        }

         /// <summary>
         /// Publish a test 
         /// </summary>
         /// <returns>Validation result - An empty list points to a sucessfull publish</returns>
        public List<string> Publish()
        {
            List<string>validationResult = GetValidationResult();

            if (validationResult.Count == 0)
            {
                using (DaoTransaction transaction = new DaoTransaction())
                {
                    try
                    {
                        this.editStatus = new EditStatus(EditStatus.Published);

                        this.Save();

                        IQuestionDAO questionDao = DaoProvider.GetQuestionDao();
                        questionDao.UpateEditStatus(transaction, this.testId.Value, new EditStatus(EditStatus.Published).ToValue());

                        transaction.Commit();
                    }
                    catch (Exception)
                    {
                        transaction.Rollback();
                        throw;
                    }
                }
            }
            return validationResult;
        }

         /// <summary>
         /// Correct the tests.
         /// Calculation are perform always in base 100 and weighted after to the test evaludation type.
         /// </summary>
         /// <returns></returns>
        public string Correct()
        {
            float score = 0;
            string scoreEvaluationType = "";

            if (questionList.Count > 0)
            {
                foreach (Question question in questionList)
                {
                    question.Correct();
                    score += question.Result.Value;
                }
                score = (score  * 100) / questionList.Count;

                scoreEvaluationType = GetScoreEvaluationType(score);             
            }

            return scoreEvaluationType;
        }

         public string GetScoreEvaluationType(float score)
        {
            string scoreEvaluationType = "";

            if (this.evaluationType.Value == EvaluationType.OneToHoundred)
            {
                scoreEvaluationType = (score).ToString("0.00");
            }
            else
            {
                if (this.evaluationType.Value == EvaluationType.AtoD)
                {
                    scoreEvaluationType = EvaluationType.GetAtoDScore(score);
                }
                else
                {
                    scoreEvaluationType = (score / 10).ToString("0.00");
                }
            }

            return scoreEvaluationType;
        }

    }
}
