﻿using System;
using System.Data;
using System.Data.Common;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AutoProofreader.Utils;
using AutoProofreader.Dao;
using AutoProofreader.Exceptions;
using AutoProofreader.Enums;
using AutoProofreader.Security;
using AutoProofreader.Email;

namespace AutoProofreader.BusinessObjects
{
    /// <summary>
    /// User business object implementation 
    /// </summary>
    [Serializable]
    public class User : BaseBusinessObject, IBusinessObject
    {
        [NonSerialized]
        private IUserDAO userDao = DaoProvider.GetUserDao();

        #region Properties
         
        private long? userId;

        public long? UserId
        {
            get { return userId; }
            set { userId = value; }
        }
        private string userName;

        public string UserName
        {
            get { return userName; }
            set { userName = value; }
        }

        private UserType userType;

        public UserType UserType
        {
            get { return userType; }
            set { userType = value; }
        }
        private string firstName;

        public string FirstName
        {
            get { return firstName; }
            set { firstName = value; }
        }
        private string lastName;

        public string LastName
        {
            get { return lastName; }
            set { lastName = value; }
        }
     
        #endregion 

        /// <summary>
        /// Default constructor
        /// </summary>
        public User()
        { 
        
        }

        /// <summary>
        /// Instatiates a user from a given User session
        /// </summary>
        public User(UserSession userSession)
        {
            this.userId = userSession.UserId;
            this.firstName = userSession.FirstName;
            this.lastName = userSession.LastName;
            this.userId = userSession.UserId;
            this.userName = userSession.UserName;
            this.userType = userSession.UserType;
        }

        /// <summary>
        /// Checks whether the business object already exists on the system
        /// </summary>
        /// <returns>True if the underlaying business object primary key has been set up</returns>
        public bool Exists()
        {
            return this.UserId.HasValue;
        }

        /// <summary>
        /// Gets user full name
        /// </summary>
        public string FullName
        {
            get
            {
                if (userId.HasValue)
                {
                    return this.lastName + ", " +  this.firstName;
                }
                else
                {
                    return "";
                }
            }
        }

        /// <summary>
        /// Loads the business object by its underlaying primary key
        /// </summary>
        /// <returns>True if the business object has been found</returns>        
        public bool Load()
        {
            if (this.Exists())
            {
                IDataReader dataReader = userDao.GetById(this.userId.Value);
                return base.Load(dataReader);            
            }
            return false;
        }

        /// <summary>
        /// Loads a user by its username
        /// </summary>
        /// <returns>True if the business object has been found</returns>        
        public bool Load(string userName)
        {
            IDataReader dataReader = userDao.GetByUserName(userName);
            return base.Load(dataReader);
        }

        /// <summary>
        /// Loads a user by its username and password
        /// </summary>
        /// <returns>True if the business object has been found</returns>  
        public bool Load(string userName, string password)
        {
            IDataReader dataReader = userDao.GetByUserNameAndPassword(userName, password);
            return base.Load(dataReader);
        }


        /// <summary>
        /// Saves the business object to the database.
        /// If the business object already exists, it will update it, whereas if it doesn't it will add it
        /// </summary>
        public void Save()
        {
            if (IsValidUserName())
            {
                if (! Exists())
                {
                    this.IsDeleted = false;

                    string password = SecurityProvider.GenerateRandomPassword();

                    using (DaoTransaction transaction = new DaoTransaction())
                    {
                        try
                        {
                            //Create user
                            this.UserId = userDao.Add(transaction, this.userName, CryptoProvider.Encrypt(password), this.userType.ToValue(), this.firstName, this.lastName, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                            //Log message
                            Log.Add(transaction, string.Format("Ha creat l'usuari {0}", this.ToString()));

                            //Commit transaction
                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                            throw;
                        }
                    }
                    //Send email with password to user
                    EmailProvider.SendCretedUserEmail(this.userName, this.firstName, this.lastName, password);
                }
                else
                {
                    using (DaoTransaction transaction = new DaoTransaction())
                    {
                        try
                        {
                            //Update user
                            userDao.Update(transaction, this.userId.Value, this.userName, this.userType.ToValue(), this.firstName, this.lastName, DateTime.Now, securityProvider.GetCurrentUserSession().UserId, this.IsDeleted);

                            //Log message
                            Log.Add(transaction, string.Format("Ha modificat l'usuari {0}", this.ToString()));

                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                            throw;
                        }
                    }
                }
            }
            else
            {
                throw new BusinessLogicException("Ja existeix un usuari amb aquest nom d'usuari");
            }
        }

        /// <summary>
        /// Checks that the user's username doesn't exist previously on the system
        /// </summary>
        /// <returns></returns>
        private bool IsValidUserName()
        {
            User user = new User();
            if (user.Load(this.userName))
            {
                return (user.UserId == this.UserId);
            }

            return true;
        }

        /// <summary>
        /// Returns business object representation as a string 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.lastName + ", " + this.FirstName;
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="dataReader">the data reader to read from</param>
        public override void FromDataReader(IDataReader dataReader)
        {
            User.FromDataReader(this, dataReader);
        }

        /// <summary>
        /// Fills business object properties from a data reader
        /// </summary>
        /// <param name="user">Business object</param>
        /// <param name="dataReader">the data reader to read from</param>
        public static void FromDataReader(User user, IDataReader dataReader)
        {
            user.UserId = DbUtils.ReadLong(dataReader, "userId");
            user.FirstName = DbUtils.ReadString(dataReader, "FirstName");
            user.LastName = DbUtils.ReadString(dataReader, "LastName");
            user.UserName = DbUtils.ReadString(dataReader, "UserName");
            user.UserType = UserType.FromString(DbUtils.ReadString(dataReader, "UserType"));

            BaseBusinessObject.FromDataReader(user, dataReader);
        }

        /// <summary>
        /// Resets user password by generating a new, random one.
        /// An email is sent to the user email address after
        /// </summary>
        public void ResetPassword()
        {
            string password = SecurityProvider.GenerateRandomPassword();

            using (DaoTransaction transaction = new DaoTransaction())
            {
                try
                {
                    //Update password
                    userDao.UpdatePassword(transaction, this.userId.Value, CryptoProvider.Encrypt(password), DateTime.Now, securityProvider.GetCurrentUserSession().UserId);

                    //Log message
                    Log.Info(transaction, string.Format("Ha actualitzat la contrasenya de l'usuari {0}", this.ToString()));

                    //Commit transaction
                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    throw;
                }
            }
            //Send email with password to user
            EmailProvider.SendPasswordEmail(this.userName, this.firstName, this.lastName, password);
        }
    }
}
