﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using AutoProofreader.BusinessObjects;
using AutoProofreader.Enums;
using AutoProofreader.Exceptions;
using AutoProofreader.Security;

namespace AutoProofreader.Resources
{
    /// <summary>
    /// Provides access to multimedia resources, images and sounds.
    /// The resources are stored on a folder within the web application
    /// </summary>
    public class ResourceProvider
    {
        private const string IMAGES_FOLDER = "Images";
        private const string SOUND_FOLDER = "Sounds";
        private const int MAX_FILE_SIZE_BYTES = 4194304;
        private static string[] validImages = { ".gif", ".png", ".jpeg", ".jpg" };
        private static string[] validSounds = { ".wav", ".mp3" };

        public static void UploadImage(string questionId, string fileName, Stream stream)
        {
            CheckFileSize(fileName, stream);

            if (IsValidExtension(fileName, IMAGES_FOLDER))
            {
                UploadFile(questionId, fileName, IMAGES_FOLDER, stream);
            }
            else
            {
                throw new BusinessLogicException(string.Format("Format d'imatge no vàlid. Només es permeten els formats: {0}", string.Join(",", validImages)));
            }
        }

        public static void UploadSound(string questionId, string fileName, Stream stream)
        {
            CheckFileSize(fileName, stream);

            if (IsValidExtension(fileName, SOUND_FOLDER))
            {
                UploadFile(questionId, fileName, SOUND_FOLDER, stream);
            }
            else
            {
                throw new BusinessLogicException(string.Format("Format de so no vàlid. Només es permeten els formats: {0}", string.Join(",", validSounds)));
            }
        }

        private static void CheckFileSize(string filename, Stream stream)
        {
            if (stream.Length > MAX_FILE_SIZE_BYTES)
            {
                throw new BusinessLogicException(string.Format("L'arxiu {0} és massa gran, el tamany màxim de l'arxiu permès és de {1} bytes. Si us plau sel.leccioni un arxiu més petit", filename, MAX_FILE_SIZE_BYTES));
            }
        }


        public static void RemoveImage(string questionId, string imageName)
        {
            RemoveFile(questionId, IMAGES_FOLDER, imageName);
        }

        public static void RemoveSound(string questionId, string imageName)
        {
            RemoveFile(questionId, SOUND_FOLDER, imageName);
        }

        public static string GetImagePath(string questionId, string imageName)
        {
            return GetRelativeFileName(questionId, IMAGES_FOLDER, imageName);
        }

        /// <summary>
        /// Returns URL for client side sound playing
        /// </summary>
        /// <param name="questionId"></param>
        /// <param name="imageName"></param>
        /// <returns></returns>
        public static string GetSoundPath(string questionId, string imageName)
        {
            string soundPath =  GetRelativeFileName(questionId, SOUND_FOLDER, imageName);
            soundPath = VirtualPathUtility.ToAbsolute(soundPath);

            return soundPath;
        }

        private static void UploadFile(string questionId, string fileName, string folder, Stream stream)
        {
            string resourcePath = GetFolderPath( questionId, folder);

            if (! Directory.Exists(resourcePath))
            {
                Directory.CreateDirectory(resourcePath);
            }
            resourcePath = Path.Combine(resourcePath, fileName);
           
            try
            {
                using (FileStream fileStream = File.Create(resourcePath))
                {
                    stream.Seek(0, SeekOrigin.Begin);
                    stream.CopyTo(fileStream);
                    fileStream.Close();
                    stream.Close();
                }
            }
            catch(Exception ex)
            {
                throw new Exception("Error guardant arxius multimèdia", ex);
            }
        }

        private static void RemoveFile(string questionId, string folder, string fileName)
        {
            string resourcePath = GetFolderPath(questionId, folder);

            DirectoryInfo files= new DirectoryInfo(resourcePath);

            foreach (FileInfo file in files.GetFiles())
            {
                file.Delete();
            }
        }

        private static string GetRelativeFileName(string questionId, string folder, string fileName)
        {
            string relativePathName = null;
            string resourcePath = GetFolderPath(questionId, folder);

            if (Directory.Exists(resourcePath))
            {
                relativePathName= Path.Combine(GetFolderRelativePath(questionId, folder), fileName);
            }
            return relativePathName;
        }

        private static string GetFolderRelativePath(string questionId, string folder)
        {
            string resourcePath = ConfigurationManager.AppSettings["ResourcesPath"];
            resourcePath = Path.Combine(resourcePath, folder);
            resourcePath = Path.Combine(resourcePath, questionId);

            return resourcePath;
        }


        private static string GetFolderPath(string questionId, string folder)
        {
            string resourcePath = GetFolderRelativePath(questionId, folder);
            resourcePath = HttpContext.Current.Server.MapPath(resourcePath);

            return resourcePath;
        }


        private static bool IsValidExtension(string fileName, string folder)
        {
            string fileExtension = Path.GetExtension(fileName).ToLower();
            string[] validExtensions;

            if (folder == IMAGES_FOLDER)
            {
                validExtensions = validImages;
            }
            else
            {
                validExtensions = validSounds;
            }

            for (int i = 0; i < validExtensions.Length; i++)
            {
                if (fileExtension == validExtensions[i])
                {
                    return true;
                }
            }
            return false;
        }
    }
}
